#ifndef Yafray_Configurable_Header
#define Yafray_Configurable_Header

#include <yafray_config.h>

#include <yafray/core/extern/vector3d.h>
#include <yafray/core/extern/color.h>

#include <map>
#include <string>
#include <list>

__BEGIN_YAFRAY

class scene_t;

struct strless
{
  bool operator()(const char* s1, const char* s2) const
	{
		return strcmp(s1, s2) < 0;
	}
};
					
class YAFRAYCORE_EXPORT configurable_t
{
	public:

		virtual bool set(const char *attr,GFLOAT value);
		virtual bool set(const char *attr,bool value);
		virtual bool set(const char *attr,int value);
		virtual bool set(const char *attr,const char *str);
		virtual bool set(const char *attr,const color_t &color);
		virtual bool set(const char *attr,const vector3d_t &vec);
		virtual bool set(const char *attr,const point3d_t &p);

		virtual bool push(const char *attr,GFLOAT value);
		virtual bool push(const char *attr,bool value);
		virtual bool push(const char *attr,int value);
		virtual bool push(const char *attr,const char *str);
		virtual bool push(const char *attr,const color_t &color);
		virtual bool push(const char *attr,const vector3d_t &vec);
		virtual bool push(const char *attr,const point3d_t &p);

		//virtual void init(scene_t &scene)=0;
		virtual ~configurable_t() {};
	protected:

		void regist(const char *attr,GFLOAT &value);
		void regist(const char *attr,bool &value);
		void regist(const char *attr,int &value);
		void regist(const char *attr,std::string &str);
		void regist(const char *attr,color_t &color);
		void regist(const char *attr,vector3d_t &vec);
		void regist(const char *attr,point3d_t &p);

		void regist(const char *attr,std::list<GFLOAT> &value);
		void regist(const char *attr,std::list<bool> &value);
		void regist(const char *attr,std::list<int> &value);
		void regist(const char *attr,std::list<std::string> &str);
		void regist(const char *attr,std::list<color_t> &color);
		void regist(const char *attr,std::list<vector3d_t> &vec);
		void regist(const char *attr,std::list<point3d_t> &p);

	private:
		
		std::map<const char *,GFLOAT *, strless>      float_map;
		std::map<const char *,bool *, strless>        bool_map;
		std::map<const char *,int *, strless>         int_map;
		std::map<const char *,std::string *, strless> str_map;
		std::map<const char *,color_t *, strless>     color_map;
		std::map<const char *,vector3d_t *, strless>  vector_map;
		std::map<const char *,point3d_t *, strless>   point_map;

		std::map<const char *,std::list<GFLOAT> *, strless>      float_list_map;
		std::map<const char *,std::list<bool> *, strless>        bool_list_map;
		std::map<const char *,std::list<int> *, strless>         int_list_map;
		std::map<const char *,std::list<std::string> *, strless> str_list_map;
		std::map<const char *,std::list<color_t> *, strless>     color_list_map;
		std::map<const char *,std::list<vector3d_t> *, strless>  vector_list_map;
		std::map<const char *,std::list<point3d_t> *, strless>   point_list_map;
};

__END_YAFRAY

#endif
