#ifndef __RENDER_H
#define __RENDER_H

#ifdef HAVE_CONFIG_H
#include<yafray_config.h>
#endif

#include <yafray/core/extern/scene.h>
#include <yafray/core/extern/output.h>
#include <yafray/core/intern/camera.h>
#include <yafray/core/intern/buffer.h>
#include <yafray/core/intern/path.h>

__BEGIN_YAFRAY

class renderer_t
{
	public:
		renderer_t(int w, int h);
		virtual ~renderer_t();
		virtual int renderABit(const scene_t &scene, colorOutput_t *out,int where) = 0;
		void setView(const point3d_t &from, const point3d_t &to, const vector3d_t &up);

	protected:
		CFLOAT  divergence(int x, int y)const;
		void calcErrors();
		light_t *chooseLight(const scene_t &scene)
		{
			if(scene.lights.empty()) return NULL;
			int tentative = ourRandomI() % scene.lights.size();
			if((last_light != NULL) && (tentative >=last_light_index))
				tentative = (tentative + 1) % scene.lights.size();
			light_t * neu = scene.lights[tentative];
			if(last_light == NULL)
			{
				last_light = neu;
				return neu;
			}
			CFLOAT f = neu->luminance() / last_light->luminance();
			if(ourRandom() < f) 
			{
				last_light = neu;
				last_light_index = tentative;
				return neu;
			}
			else return last_light;
		};
		void collectLightEnergy(const scene_t &scene);
		
		int width,height;
		Buffer_t<color_t> image;
		Buffer_t<CFLOAT> weight;
		Buffer_t<CFLOAT> error;
		CFLOAT max_error;

		camera_t *camera;
		CFLOAT total_luminance;
		light_t * last_light;
		int last_light_index;
		int resampled;
};

class lafortune_t : public renderer_t
{
	public:
		lafortune_t(int w, int h);
		virtual ~lafortune_t();
		virtual int renderABit(const scene_t &scene, colorOutput_t *out,int where);

	protected:
		color_t biPathTrace(const scene_t &scene, int x, int y, int samples);
		color_t calcLight(const lightPath_t &lpath, const eyePath_t &epath,
											const scene_t &scene)const;
		
};

class veach_t : public renderer_t
{
	public:
		veach_t(int w, int h);
		virtual ~veach_t();
		virtual int renderABit(const scene_t &scene, colorOutput_t *out,int where);

	protected:
		PFLOAT totalPathProb(const lightPath_t &lpath, const eyePath_t &epath)const;
		color_t biPathTrace(const scene_t &scene, int x, int y, int samples);
		color_t calcLight(const lightPath_t &lpath, const eyePath_t &epath,
											const scene_t &scene)const;
		
};

__END_YAFRAY

#endif
