#ifndef __SCENE_H
#define __SCENE_H

#ifdef HAVE_CONFIG_H
#include<yafray_config.h>
#endif

#include <yafray/core/extern/light.h>
#include <yafray/core/extern/triangle.h>
#include <yafray/core/extern/bound.h>
#include <yafray/core/extern/surface.h>
#include <yafray/core/extern/matrix4.h>
#include <vector>
#include <list>

__BEGIN_YAFRAY

class scene_t
{
	friend class renderer_t;
	public:
		scene_t();
		~scene_t();

		// LOAD
		void startGeometry();
		void startObject();

		int addVertex(const point3d_t &p);
		int addOrco(const point3d_t &p);
		int addColor(const color_t &c);
		
		void addTriangle(int pa,int pb,int pc,
										 const vector3d_t &na,const vector3d_t &nb,const vector3d_t &nc);
		void setOrco(int oa,int ib,int oc); //For last added triangle
		void setUV(GFLOAT uv[3][2]);        //For last added triangle
		void setUV(GFLOAT a_u,GFLOAT a_v,GFLOAT b_u,GFLOAT b_v,GFLOAT c_u,GFLOAT c_v);
		void setColor(int ca,int cb,int cc);//For last added triangle
		
		void endObject();
		void endGeometry();

		configurable_t *addLight(const char *type);

		// RENDER
		bool shoot(const ray_t &ray,surfacePoint_t &sp, const void *skip=NULL)const;
		bool shadow(const segment_t &seg, const void *skip1 = NULL,
																			const void *skip2 = NULL)const;
	protected:
		void calculateBound();
		struct // Geometry data
		{
			std::vector<point3d_t> points;
			std::vector<point3d_t> orcos;
			std::vector<color_t> colors;
			std::vector<triangle_t> triangles;
			std::vector<std::pair<int,int> > objects; // begin and end of an object triangle set
			void *tree;
			bound_t bound;
		}geometry;
		std::vector<light_t *> lights;

		struct // Load time data
		{
			int object; // object being created id 
			int tribegin;  // first triangle of the object
			int pbegin; // first point of the object
			int obegin; // first orco of the object
			int cbegin; // first color of the object
			triangle_t *lasttri; // last added triangle
			std::list<matrix4x4_t> matrix_stack; // transformation stack
		}load;

		struct state
		{
			enum
			{
				NONE,
				GEOMETRY,
				OBJECT
			};
		};
		std::list<int> state_stack;
		int getState()const {return state_stack.front();};
};

__END_YAFRAY

#endif
