/****************************************************************************
 *
 * 			triangle.h: Face representation and manipulation api 
 *      This is part of the yafray package
 *      Copyright (C) 2002 Alejandro Conty Estevez
 *
 *      This library is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU Lesser General Public
 *      License as published by the Free Software Foundation; either
 *      version 2.1 of the License, or (at your option) any later version.
 *
 *      This library is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public
 *      License along with this library; if not, write to the Free Software
 *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *      
 */
#ifndef __TRIANGLE_H
#define __TRIANGLE_H

#ifdef HAVE_CONFIG_H
#include<yafray_config.h>
#endif

#include <yafray/core/extern/surface.h>
#include <yafray/core/extern/vector3d.h>

#include <vector>

__BEGIN_YAFRAY

class YAFRAYCORE_EXPORT triangle_t
{
	public:
		// CREATION TIME
		triangle_t(int va,int vb,int vc);
		triangle_t();
		~triangle_t() {};
		void setVertices(int va,int vb,int vc);
		void recNormal();
		const vector3d_t & N() const {return normal;};


		void setNormals(const vector3d_t &_na,const vector3d_t &_nb,const vector3d_t &_nc)
			{na=_na;nb=_nb;nc=_nc;};
		void setUV(GFLOAT a_u,GFLOAT a_v,GFLOAT b_u,GFLOAT b_v,GFLOAT c_u,GFLOAT c_v);
		void setVCOL(int a,int b,int c);
		void setOrco(int a,int b,int c);
		void setObject(int o) {object=o;};

		static void setContext(const std::vector<point3d_t> &P,const std::vector<point3d_t> &O,
										const std::vector<color_t> &C) 
			{points=&P;orcos=&O;colors=&C;};

		// RENDER TIME
		bool itsZP()const {return (normal.z==0);};
		bool itsYP()const {return (normal.y==0);};
		bool itsXP()const {return (normal.x==0);};
		int getObject()const {return object;};
	
		const point3d_t & pointA()const {return (*points)[pa];};
		const point3d_t & pointB()const {return (*points)[pb];};
		const point3d_t & pointC()const {return (*points)[pc];};

		bool hit(const ray_t &ray)const
		{
			const point3d_t &a=(*points)[pa],&b=(*points)[pb],&c=(*points)[pc];
			vector3d_t va=a-ray.from,vb=b-ray.from,vc=c-ray.from,r;
			if((ray.dir*normal)<0) r=-ray.dir;
			else r=ray.dir;
			if( ((va^vb)*r)<0 ) return false;
			if( ((vb^vc)*r)<0 ) return false;
			if( ((vc^va)*r)<0 ) return false;
			return true;
		}
		
		PFLOAT intersect(const ray_t &ray)const
		{
			const point3d_t &a=(*points)[pa];
			return (normal*(a-ray.from))/(normal*ray.dir);
		}
		surfacePoint_t  getSurface(point3d_t &h,PFLOAT d)const;

	protected:
		void check()const;

		static const std::vector<point3d_t>  *points;
		static const std::vector<point3d_t>  *orcos;
		static const std::vector<color_t>    *colors;

		int object;
		int pa,pb,pc;
		int oa,ob,oc;
		vector3d_t na,nb,nc;
		GFLOAT uv[3][2];
		int ca,cb,cc;
		bool hasuv, has_vcol,has_orco;
		vector3d_t normal;
};

__END_YAFRAY
#endif
