/****************************************************************************
 *
 * 			buffer.h: Buffers (color and float) api
 *      This is part of the yafray package
 *      Copyright (C) 2002  Alejandro Conty Estvez
 *
 *      This library is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU Lesser General Public
 *      License as published by the Free Software Foundation; either
 *      version 2.1 of the License, or (at your option) any later version.
 *
 *      This library is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public
 *      License along with this library; if not, write to the Free Software
 *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __BUFFER_H
#define __BUFFER_H

#include<yafray/core/extern/tools.h>

#ifdef HAVE_CONFIG_H
#include<yafray_config.h>
#endif


__BEGIN_YAFRAY

template<typename T1, unsigned char T2 = 1>
class YAFRAYCORE_EXPORT Buffer_t
{
	public :
		Buffer_t(int x,int y)
		{
			data = _tn(new T1 [x*y*T2]);
			if (data==NULL)
			{
				std::cerr << "Error allocating memory in cBuffer\n";
				exit(1);
			}
			mx = x;
			my = y;
		}

		Buffer_t() { data=NULL; }
		~Buffer_t() { if (data) delete[] _td(data); }

		void set(int x,int y)
		{
			if (data) delete[] _td(data);
			data = _tn(new T1 [x*y*T2]);
			if(data==NULL)
			{
				std::cerr << "Error allocating memory in cBuffer\n";
				exit(1);
			}
			mx = x;
			my = y;
		}

		T1 * operator()(int x,int y) 
		{
			return &data[(y*mx+x)*T2]; 
		}

		const T1 * operator()(int x,int y)const
		{
			return &data[(y*mx+x)*T2]; 
		}

		Buffer_t & operator = (const Buffer_t &source)
		{
			if ((mx!=source.mx) || (my!=source.my)) std::cerr << "Error, trying to assign buffers of a different size\n";
			if ((data == NULL) || (source.data == NULL)) std::cerr << "Assigning unallocated buffers\n";
			int total = mx*my*T2;
			for(int i=0;i<total;++i)
				data[i] = source.data[i];
			return *this;
		}

		int width() const { return mx; };
		int height() const { return my; };
	protected :
		T1 *data;
		int mx, my;
};

__END_YAFRAY

#endif
