#ifndef __PATH_H
#define __PATH_H

#ifdef HAVE_CONFIG_H
#include <yafray_config.h>
#endif

#include <yafray/core/extern/vector3d.h>
#include <yafray/core/extern/scene.h>
#include <yafray/core/extern/color.h>
#include <yafray/core/extern/light.h>
#include <yafray/core/extern/mcqmc.h>
#include <vector>

__BEGIN_YAFRAY

class eyePath_t
{
	public:
		struct node_t
		{
			node_t(const color_t &R, const color_t &F, const point3d_t &P, const vector3d_t &_N, 
						CFLOAT rhp, PFLOAT brdf, void *O):
				orradiance(R), filter(F), position(P), N(_N), 
				trans_prob(0), rtrans_prob(0), hit_prob(0) , rhit_prob(rhp),
				BRDF(brdf), diffuse_jump(true), origin(O)
				{};

			color_t orradiance;
			color_t filter;
			point3d_t position;
			vector3d_t N;
			CFLOAT trans_prob, rtrans_prob;
			CFLOAT hit_prob, rhit_prob;
			CFLOAT BRDF;
			bool diffuse_jump;
			void *origin;
		};

		void add(const color_t &I, const color_t &F, const point3d_t &P, const vector3d_t &N, 
						CFLOAT rhp, PFLOAT brdf, void *O)
		{
			_path.push_back(node_t(I, F, P, N, rhp, brdf, O));
		};
		
		const std::vector<node_t> &path()const {return _path;};
		std::vector<node_t> &path() {return _path;};
		
		void reset() {_path.clear();};

		static void createPath(const scene_t &sc, const ray_t & ray, eyePath_t &path, int maxdepth);
		
	protected:
		std::vector<node_t> _path;
};

class lightPath_t
{
	public:
		struct node_t
		{
			node_t(const color_t &R, const color_t &F, const point3d_t &P, const vector3d_t &_N, 
						CFLOAT hp, PFLOAT brdf, void *O):
				irradiance(R), filter(F), position(P), N(_N), 
				trans_prob(0), rtrans_prob(0), hit_prob(hp) , rhit_prob(0),
				BRDF(brdf), diffuse_jump(true), origin(O)
				{};

			color_t irradiance;
			color_t filter;
			point3d_t position;
			vector3d_t N;
			CFLOAT trans_prob, rtrans_prob;
			CFLOAT hit_prob, rhit_prob;
			CFLOAT BRDF;
			bool diffuse_jump;
			void *origin;
		};

		void add(const color_t &I, const color_t &F, const point3d_t &P, const vector3d_t &N, 
						CFLOAT hp, PFLOAT brdf, void *O)
		{
			_path.push_back(node_t(I, F, P, N, hp, brdf, O));
		};
		
		const std::vector<node_t> &path()const {return _path;};
		std::vector<node_t> &path() {return _path;};
		
		void reset() {_path.clear();};

		static void createPath(const scene_t &sc, light_t *L, lightPath_t &path, int maxdepth);
		
	protected:
		std::vector<node_t> _path;
};

__END_YAFRAY

#endif
