/****************************************************************************
 *
 * 			meshtree.cc: mesh bsp creation
 *      This is part of the yafray package
 *      Copyright (C) 2002  Alejandro Conty Estvez
 *
 *      This library is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU Lesser General Public
 *      License as published by the Free Software Foundation; either
 *      version 2.1 of the License, or (at your option) any later version.
 *
 *      This library is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public
 *      License along with this library; if not, write to the Free Software
 *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


using namespace std;
#include <yafray/core/extern/tools.h>
#include <yafray/core/intern/meshtree.h>
#include <yafray/core/intern/triangletools.h>

__BEGIN_YAFRAY

PFLOAT mt_foo1,mt_foo2;


void * buildTriangleTree(std::vector<triangle_t*> *v,
		const bound_t &bound,unsigned int dtol,unsigned int depth,unsigned int lostd,
		PFLOAT &avgdepth,PFLOAT &avgsize)
{
	if((v->size()<=dtol) || (lostd>3))
	{
		avgdepth=(PFLOAT)depth;
		avgsize=(PFLOAT)v->size();
		return _tn(new tnode_t(v));
	}
	int axis;
	PFLOAT media;
	PFLOAT lx,ly,lz;
	lx=bound.longX();
	ly=bound.longY();
	lz=bound.longZ();

	bound_t bl=bound,br=bound;
	plane_t plane;
	if((lx>=ly) && (lx>=lz))
	{
		media=bound.centerX();
		bl.setMaxX(media);
		br.setMinX(media);
		axis=plane_t::XAXIS;
	}
	else if( (ly>=lz) )
	{
		media=bound.centerY();
		bl.setMaxY(media);
		br.setMinY(media);
		axis=plane_t::YAXIS;
	}
	else
	{
		media=bound.centerZ();
		bl.setMaxZ(media);
		br.setMinZ(media);
		axis=plane_t::ZAXIS;
	}
	plane.set(bound,media,axis);
		
	std::vector<triangle_t *> *vl=_tn(new vector<triangle_t *>()),*vr=_tn(new vector<triangle_t *>());
	v_foreach_const(triangle_t *, i, *v)
	{
		int pos=cheapPosition(**i,bound,media,axis);
		if(pos==trianglePosition_t::NONE) pos=expensivePosition(**i,bound,media,axis);
		if(pos==trianglePosition_t::LOWER) vl->push_back(*i);
		else if(pos==trianglePosition_t::HIGHER) vr->push_back(*i);
		else //INTERSECT
		{
			vl->push_back(*i);
			vr->push_back(*i);
		}
	}

	if(vl->empty() || vr->empty())
	{
		if(vl->empty())	media=minimize(*vr,bound,axis);
		else media=maximize(*vl,bound,axis);
		switch(axis)
		{
			case AXISX:
				bl.setMaxX(media);
				br.setMinX(media);
				plane.set(bound,media,plane_t::XAXIS);
				break;
			case AXISY:
				bl.setMaxY(media);
				br.setMinY(media);
				plane.set(bound,media,plane_t::YAXIS);
				break;
			case AXISZ:
				bl.setMaxZ(media);
				br.setMinZ(media);
				plane.set(bound,media,plane_t::ZAXIS);
				break;
		}
	}

	int lostl=((vl->size()==v->size())) ? (lostd+1) : (lostd-1);
	int lostr=((vr->size()==v->size())) ? (lostd+1) : (lostd-1);
	if(lostl<0) lostl=0;
	if(lostr<0) lostr=0;
	delete _td(v);
	PFLOAT avgd,avgs;
	tnode_t *left=(tnode_t *)buildTriangleTree(vl,bl,dtol,depth+1,lostl,avgd,avgs);
	avgdepth=avgd;
	avgsize=avgs;
	tnode_t *right=(tnode_t *)buildTriangleTree(vr,br,dtol,depth+1,lostr,avgd,avgs);
	avgdepth+=avgd;
	avgsize+=avgs;
	avgdepth*=0.5;
	avgsize*=0.5;

	return _tn(new tnode_t(left,right,plane));
}

__END_YAFRAY
