/****************************************************************************
 *
 * 			meshtree.cc: point light
 *      This is part of the yafray package
 *      Copyright (C) 2002  Alejandro Conty Estvez
 *
 *      This library is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU Lesser General Public
 *      License as published by the Free Software Foundation; either
 *      version 2.1 of the License, or (at your option) any later version.
 *
 *      This library is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public
 *      License along with this library; if not, write to the Free Software
 *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


using namespace std;
#include <yafray/core/extern/tools.h>
#include <yafray/core/intern/pointlight.h>

__BEGIN_YAFRAY


pointLight_t::pointLight_t():N(0,0,1),U(1,0,0),V(0,1,0)
{
	regist("color",color);
	regist("power",power);
	regist("position",position);
	HSEQ = _tn(new Halton[2]);
	HSEQ[0].setBase(2);
	HSEQ[1].setBase(3);
}

CFLOAT pointLight_t::luminance()const
{
	return color.luminance() * power;
}

void pointLight_t::getSample(const point3d_t &from, energy_t &energy)
{
	energy.segment.dir = position - from;
	PFLOAT dist = energy.segment.dir.normLen();
	energy.segment.from = from;
	energy.segment.dist = dist;
	energy.color = color * power / (4*M_PI*(dist*dist));
}

void pointLight_t::getSample(energy_t &energy)
{
	PFLOAT r1 = ourRandom()/*HSEQ[0].getNext()*/, r2 = ourRandom()/*HSEQ[1].getNext()*/;
	energy.segment.dir = randomVectorCone(N,U,V, -1.0, r1,r2); 
	energy.segment.from = position;
	energy.color = color * power;
}

__END_YAFRAY
