#include <yafray/core/extern/tools.h>
#include<yafray/core/extern/render.h>
#include <yafray/core/intern/path.h>

using namespace std;

__BEGIN_YAFRAY

renderer_t::renderer_t(int w, int h):width(w),height(h),image(w,h),weight(w,h),error(w,h)
{
	for(int y=0;y<height;y++)
		for(int x=0;x<width;x++)
		{
			*weight(x,y) = 0.0;
			*error(x,y) = 1.0;
		}
	max_error = 0.0;
	resampled = 0;
	camera = NULL;
	last_light = NULL;
}

renderer_t::~renderer_t()
{
	if(camera!=NULL) delete _td(camera);
}

void renderer_t::setView(const point3d_t &from, const point3d_t &to, const vector3d_t &up)
{
	if(camera!=NULL) delete _td(camera);
	camera = _tn(new camera_t(from,to,from + up, width, height));
}
/*
void renderer_t::render(const scene_t &scene, colorOutput_t *out)
{
	if(camera == NULL)
	{
		cout<<"Rendering without camera"<<endl;
		return;
	}

	surfacePoint_t sp;
	
	for(int y=0;y<height;y++)
	{
		for(int x=0;x<width;x++)
		{
			PFLOAT rx = x, ry = y, weight;
			ray_t ray(camera->position(), camera->shootRay(rx, ry, weight));
			if(scene.shoot(ray, sp))
			{
				sp.light.diffuse = color_t(0.9, 0.9, 0.9);
				*image(x,y) = directLight(scene, sp, - ray.dir);
			}
			else *image(x,y) = color_t(0.0);
		}
		for(int x=0;x<width;x++) out->putPixel(x, y, *image(x,y), 1.0, 1.0);
	}
}


color_t renderer_t::directLight(const scene_t &scene,const surfacePoint_t &sp, 
			const vector3d_t &eye)const
{
	color_t total(0,0,0);
	v_foreach_const(light_t *, i, scene.lights)
	{
		energy_t energy; 
		(*i)->getSample(sp.geometry.P, energy);

		vector3d_t N = FACE_FORWARD(sp.geometry.Ng,sp.geometry.N, eye);

		CFLOAT cosine = energy.segment.dir*N;
		if(cosine<0) continue;
		if(!scene.shadow(energy.segment , sp.basic.origin))
		{
			color_t diffuse = energy.color * sp.light.diffuse * cosine;
			total += diffuse;
		}
	}
	return total;
}

color_t renderer_t::calcLight(const eyePath_t &path,const scene_t &scene, light_t * L)const
{
	color_t total(0,0,0);
	energy_t energy;
	color_t _filter(1,1,1);
	const color_t *filter = &_filter;
	//TODO: emitttt

	v_foreach_const(eyePath_t::node_t, i, path.path())
	{
		L->getSample(i->position, energy);
		CFLOAT cosine = energy.segment.dir * i->N;
		if(cosine>0)
			if(!scene.shadow(energy.segment , i->origin ))
				total += *filter * energy.color * i->diffuse * cosine;
		filter = &(i->filter);
	}
	return total;
}


color_t renderer_t::pathTrace(const scene_t &scene, int x, int y, int samples)
{
	CFLOAT factor = 1.0 / (CFLOAT) samples;

	PFLOAT rx = x, ry = y, weight;
	ray_t ray(camera->position(), camera->shootRay(rx, ry, weight));
	eyePath_t path;
	color_t total(0,0,0);

	for(int i = 0; i < samples; ++i)
	{
		eyePath_t::createPath(scene, ray, path, 2);
		light_t *L = chooseLight(scene);
		if(L!=NULL)
		{
			CFLOAT prob = L->luminance() / total_luminance;
			total += calcLight(path, scene, L) / prob;
		}
	}
	return total * factor;
}
*/



CFLOAT  renderer_t::divergence(int x, int y)const
{
	color_t center = *image(x,y);
	center.clampRGB01();
	CFLOAT max=0.0;
	
	for(int yoff=-1;yoff<=1;++yoff)
		for(int xoff=-1;xoff<=1;++xoff)
		{
			int vx = x + xoff;
			int vy = y + yoff;
			if((vx<0) || (vx>=width) || (vy<0) || (vy>=height)) continue;
			color_t neig = *image(vx, vy);
			neig.clampRGB01();
			CFLOAT div = maxAbsDiff( neig, center);
			if(div>max) max = div;
		}
	return max;
}

void renderer_t::calcErrors()
{
	CFLOAT max = 0.0;
	CFLOAT avg = 0.0;
	for(int y=0;y<height;++y)
		for(int x=0;x<width;++x)
		{
			*error(x,y) = divergence(x,y);
			avg += *error(x,y);
			if(*error(x,y) > max) max = *error(x,y);
		}
	avg /= (float)width*height;
	cout<<"max "<<max<<endl;
	cout<<"avg "<<avg<<endl;
	max_error = 0.03;
}

void renderer_t::collectLightEnergy(const scene_t &scene)
{
	total_luminance = 0;
	v_foreach_const(light_t *, i, scene.lights) total_luminance += (*i)->luminance();
}

__END_YAFRAY
