
#include <yafray/core/extern/tools.h>
#include<yafray/core/extern/render.h>
#include <yafray/core/intern/path.h>

using namespace std;

__BEGIN_YAFRAY

veach_t::veach_t(int w, int h):renderer_t(w,h)
{
}

veach_t::~veach_t()
{
}

int veach_t::renderABit(const scene_t &scene, colorOutput_t *out,int where)
{
	if(camera == NULL)
	{
		cout<<"Rendering without camera"<<endl;
		return -1;
	}
	if(where <0 ) return where;
	
	if(where == 0) collectLightEnergy(scene);
	
	surfacePoint_t sp;
	
	int endline = where + 16;
	if(endline>height) endline = height;
	
	int passsamples=4;
	for(int y=where;y<endline;y++)
	{
		for(int x=0;x<width;x++)
		{
			if( *error(x,y) > max_error )
			{
				color_t color = biPathTrace(scene, x, y, passsamples);
				CFLOAT lw = *weight(x,y);
				*image(x,y) = (*image(x,y) * lw + color * (CFLOAT)passsamples) 
											/ (lw + (CFLOAT)passsamples);
				*weight(x,y) = lw + (CFLOAT)passsamples;
				resampled ++;
			}
		}
		for(int x=0;x<width;x++) out->putPixel(x, y, *image(x,y), 1.0, 1.0);
	}

	if(endline == height) 
	{
		cout<<"Resampled "<< 100.0*(float)resampled/((float)width*height) << "%% pixels" <<endl;
		calcErrors();
		resampled = 0;
		return -1; 
	}
	else 
		return endline;
}

	struct simpleNode_t
	{
		CFLOAT trans_prob, rtrans_prob, hit_prob, rhit_prob;
		bool diffuse;
	};
	
PFLOAT veach_t::totalPathProb(const lightPath_t &lpath, const eyePath_t &epath)const
{
	const eyePath_t::node_t & S   = epath.path().back();
	const lightPath_t::node_t & T = lpath.path().back();

	vector3d_t ldir = T.position - S.position;
	PFLOAT dist = ldir.normLen();
	CFLOAT icos = ldir * S.N;
	CFLOAT ocos = - (ldir * T.N);

	vector<simpleNode_t> path(epath.path().size() + lpath.path().size());

	int j = 0;
	int iS,iT;
	v_foreach_const(lightPath_t::node_t, i, lpath.path())
	{
		path[j].trans_prob = i->trans_prob;
		path[j].rtrans_prob = i->rtrans_prob;
		path[j].hit_prob = i->hit_prob;
		path[j].rhit_prob = i->rhit_prob;
		path[j].diffuse = i->diffuse_jump;
		j++;
	}
	iT = j-1;
	iS = j;
	for(vector<eyePath_t::node_t>::const_reverse_iterator i= epath.path().rbegin(); 
				i != epath.path().rend(); i++)
	{
		path[j].trans_prob = i->trans_prob;
		path[j].rtrans_prob = i->rtrans_prob;
		path[j].hit_prob = i->hit_prob;
		path[j].rhit_prob = i->rhit_prob;
		path[j].diffuse = i->diffuse_jump;
		j++;
	}
	if(lpath.path().size() > 1)
	{
		path[iT].trans_prob = T.filter.energy() * ocos / M_PI;
		path[iT].rhit_prob = ocos / (dist * dist);
	}
	else
	{
		path[iT].trans_prob = 1.0 / (4.0 * M_PI);
		path[iT].rhit_prob = 0;
	}
	path[iS].rtrans_prob = S.filter.energy() * icos / M_PI;
	path[iS].hit_prob = icos / (dist * dist);

	int depth = path.size() - 1;
	CFLOAT total = 0.0;
	CFLOAT meto_prob = 1.0 / (CFLOAT)depth;
	
	//cout<<"Empezamos:"<<endl;
	for(int i=0; i < depth ; ++i)
		if( path[i].diffuse && path[i+1].diffuse ) 
		{
			CFLOAT prob = 1.0;
			CFLOAT jump_prob = 1.0;
			for(int j=0;j<=i;j++)
			{
				prob *= jump_prob * path[j].hit_prob;
				jump_prob = path[j].trans_prob;
			}
			jump_prob = 1.0;
			for(int j=path.size() - 1; j>=(i+1); --j)
			{
				prob *= jump_prob * path[j].rhit_prob;
				jump_prob = path[j].rtrans_prob;
			}
			//if(i == (lpath.path().size()-1))
			//	cout<<prob<<" <-"<<endl;
			//else
			//	cout<<prob<<endl;
			total += prob * meto_prob;
		}
	//cout <<"total: "<<total<<" por "<<meto_prob<<endl;
	//getchar();
	return total;
}

color_t veach_t::calcLight(const lightPath_t &lpath, const eyePath_t &epath,
															const scene_t &scene)const
{
	int ldepth = lpath.path().size() - 1;
/*
	CFLOAT path_prob = 1.0;
	CFLOAT jump_prob = 1.0;
	v_foreach_const(eyePath_t::node_t, i, epath.path())
	{
		path_prob *= jump_prob * i->rhit_prob;
		jump_prob = i->rtrans_prob;
	}

	jump_prob = 1.0;
	v_foreach_const(lightPath_t::node_t, i, lpath.path())
	{
		path_prob *= jump_prob * i->hit_prob;
		jump_prob = i->trans_prob;
	}
*/

	const eyePath_t::node_t & S   = epath.path().back();
	const lightPath_t::node_t & T = lpath.path().back();

	vector3d_t ldir = T.position - S.position;
	PFLOAT dist = ldir.normLen();
	CFLOAT icos = ldir * S.N;
	CFLOAT ocos = - (ldir * T.N);
	CFLOAT aten = ((ldepth > 0) ? ocos : 1.0) * icos / (dist * dist);
		
	if((aten<=0) || (icos<=0)) return color_t(0,0,0);
	
	CFLOAT path_prob = totalPathProb(lpath, epath);
	
	if(!scene.shadow(segment_t(T.position, S.position) , 
												S.origin, T.origin ))
		return T.irradiance * T.filter * T.BRDF
					 * S.orradiance * S.filter * S.BRDF
					 * aten / path_prob;
	return color_t(0,0,0);
}

color_t veach_t::biPathTrace(const scene_t &scene, int x, int y, int samples)
{
	CFLOAT factor = 1.0 / (CFLOAT) samples;

	PFLOAT rx = x, ry = y, weight;
	ray_t ray(camera->position(), camera->shootRay(rx, ry, weight));
	eyePath_t epath;
	lightPath_t lpath;
	color_t total(0,0,0);
	const int maxdepth = 4;

	for(int i = 0; i < samples; ++i)
	{
		light_t *L = chooseLight(scene);
		if(L!=NULL)
		{
			CFLOAT prob = L->luminance() / total_luminance;
			int depth =  ourRandomI() % (maxdepth +1);
			int edepth = ourRandomI() % (depth +1);
			int ldepth = depth - edepth;
			prob *= 1.0 / (CFLOAT) (maxdepth +1); // the prob of the path length
			eyePath_t::createPath(scene, ray, epath, edepth);
			lightPath_t::createPath(scene, L, lpath, ldepth);
			total += calcLight(lpath, epath, scene) / prob;
		}
	}
	return total * factor;
}

__END_YAFRAY
