#include <yafray/fry/manager.h>
#include <yafray/fry/draw.h>
#include <yafray/core/extern/configurable.h>

using namespace yafray;
using namespace std;

void testScene(scene_t &scene, renderer_t &renderer);

renderManager_t::renderManager_t(int w, int h):renderer(w,h)
{
	state = 0;
	testScene(scene, renderer);
}

bool renderManager_t::work()
{
	state = renderer.renderABit( scene, this, state);
	painter_t::it().display();
	if(state < 0) return false;
	return true;
}
		
bool renderManager_t::putPixel(int x, int y,const color_t &c, 
				CFLOAT alpha,PFLOAT depth)
{
	color_t temp = c;
	temp.clampRGB01();
	unsigned char R = (unsigned char)(temp.R * 255.0);
	unsigned char G = (unsigned char)(temp.G * 255.0);
	unsigned char B = (unsigned char)(temp.B * 255.0);

	painter_t::it().set(x,y,R,G,B);
	return true;
}

void renderManager_t::flush()
{
}

static void cube(scene_t &scene, const point3d_t &center, PFLOAT radius)
{
	int a,b,c,d,e,f,g,h;

	a = scene.addVertex(center + vector3d_t(radius,radius,radius) );
	b = scene.addVertex(center + vector3d_t(radius,-radius,radius) );
	c = scene.addVertex(center + vector3d_t(radius,-radius,-radius) );
	d = scene.addVertex(center + vector3d_t(radius,radius,-radius) );
	
	e = scene.addVertex(center + vector3d_t(-radius,radius,-radius) );
	f = scene.addVertex(center + vector3d_t(-radius,-radius,-radius) );
	g = scene.addVertex(center + vector3d_t(-radius,-radius,radius) );
	h = scene.addVertex(center + vector3d_t(-radius,radius,radius) );

	vector3d_t N(1,0,0);
	scene.addTriangle(a, b, c, N,N,N);
	scene.addTriangle(c, d, a, N,N,N);
	N.set(-1,0,0);
	scene.addTriangle(e, f, g, N,N,N);
	scene.addTriangle(g, h, e, N,N,N);
	N.set(0,-1,0);
	scene.addTriangle(g, f, c, N,N,N);
	scene.addTriangle(g, c, b, N,N,N);
	N.set(0,1,0);
	scene.addTriangle(a, e, h, N,N,N);
	scene.addTriangle(e, a, d, N,N,N);
	N.set(0,0,1);
	scene.addTriangle(a, g, b, N,N,N);
	scene.addTriangle(g, a, h, N,N,N);
	N.set(0,0,-1);
	scene.addTriangle(d, c, f, N,N,N);
	scene.addTriangle(f, e, d, N,N,N);
}

void testScene(scene_t &scene, renderer_t &renderer)
{
	scene.startGeometry();
	scene.startObject();

	cube(scene, point3d_t(0,0,5), 5);
	cube(scene, point3d_t(0,0,5), 6);
	cube(scene, point3d_t(0,0,5), 7);
	cube(scene, point3d_t(0,0,5), 8);
	cube(scene, point3d_t(0,0,0.7), 0.7);

	scene.endObject();
	scene.endGeometry();

	configurable_t * luz = scene.addLight("point");
	luz->set("color", color_t(1.0, 1.0, 1.0));
	luz->set("power", (float)(4.0*M_PI* 4.0));
	luz->set("position", point3d_t(0,-1,1));

	luz = scene.addLight("point");
	luz->set("color", color_t(1.0, 1.0, 1.0));
	luz->set("power", (float)(4.0*M_PI* 4.0));
	luz->set("position", point3d_t(0,-1.1,1));
	luz = scene.addLight("point");
	luz->set("color", color_t(1.0, 1.0, 1.0));
	luz->set("power", (float)(4.0*M_PI* 4.0));
	luz->set("position", point3d_t(0,-1.2,1));
	luz = scene.addLight("point");
	luz->set("color", color_t(1.0, 1.0, 1.0));
	luz->set("power", (float)(4.0*M_PI* 4.0));
	luz->set("position", point3d_t(0,-1.3,1));

	
	renderer.setView(point3d_t(-4,2,2),point3d_t(0,0,1.4), vector3d_t(0,0,1));
}

