/****************************************************************************
 *
 * 			pointlight.cc: Point light implementation 
 *      This is part of the yafray package
 *      Copyright (C) 2002 Alejandro Conty Estevez
 *
 *      This library is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU Lesser General Public
 *      License as published by the Free Software Foundation; either
 *      version 2.1 of the License, or (at your option) any later version.
 *
 *      This library is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public
 *      License along with this library; if not, write to the Free Software
 *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *      
 */

#include <yafray/lights/intern/pointlight.h>
#include <limits>

__BEGIN_YAFRAY

pointLight_t::pointLight_t()
{
	regist("from",from);
	regist("color",color);
	regist("power",pow);
	regist("shadows",cast_shadows);
}

void pointLight_t::init(scene_t &scene) 
{
	color*=pow;
}

color_t pointLight_t::energy(const surfacePoint_t &sp,const scene_t &scene,CFLOAT importance)const
{
	//TODO: Shadow? Use the scene when defined
	vector3d_t dir=from-sp.P;
	CFLOAT distance=dir.normLen();
	CFLOAT cos_angle=dir*sp.N;
	if(cos_angle<=0.0) return color_t(0.0);
	if(distance==0.0) return color_t(std::numeric_limits<CFLOAT>::infinity());
	return color*(cos_angle/(distance*distance));
}

color_t pointLight_t::specular(const surfacePoint_t &sp,const vector3d_t &eye,
									const color_t &energy,CFLOAT hard)const
{
	vector3d_t dir=reflect(sp.N,from-sp.P);
	dir.normalize();
	CFLOAT cos_angle=dir*eye;
	if(cos_angle<=0.0) return color_t(0.0);
	return std::pow(cos_angle,hard)*energy;
}

color_t pointLight_t::totalEnergy()const
{
	return 4.0*M_PI*color;
}

configurable_t *pointLight_t::factory()
{
	return new pointLight_t();
}

extern "C"
{
	
YAFRAYPLUGIN_EXPORT void registerPlugin(/* renderEnvironment_t &render */)
{
	//render.registerFactory("pointlight",pointLight_t::factory);
	std::cout<<"Registered pointlight\n";
}

}
__END_YAFRAY
